@@ -245,6 +245,25 @@ mm_modem_3gpp_get_registration_state (MMModem3gpp *self)
 
 /*****************************************************************************/
 
+/**
+ * mm_modem_3gpp_get_subscription_state:
+ * @self: A #MMModem.
+ *
+ * Get the current subscription status of the account. This value is only
+ * available after the modem attempts to register with the network.
+ *
+ * Returns: A #MMModem3gppSubscriptionState value, specifying the current subscription state.
+ */
+MMModem3gppSubscriptionState
+mm_modem_3gpp_get_subscription_state (MMModem3gpp *self)
+{
+    g_return_val_if_fail (MM_IS_MODEM_3GPP (self), MM_MODEM_3GPP_SUBSCRIPTION_STATE_UNKNOWN);
+
+    return mm_gdbus_modem3gpp_get_subscription_state (MM_GDBUS_MODEM3GPP (self));
+}
+
+/*****************************************************************************/
+
 /**
  * mm_modem_3gpp_get_enabled_facility_locks:
  * @self: A #MMModem3gpp.
@@ -499,7 +518,7 @@ create_networks_list (GVariant *variant)
  *
  * Finishes an operation started with mm_modem_3gpp_scan().
  *
- * Returns: a list of #MMModem3gppNetwork structs, or #NULL if @error is set. The returned value should be freed with g_list_free_full() using mm_modem_3gpp_network_free() as #GDestroyNotify function.
+ * Returns: (transfer full) (element-type ModemManager.Modem3gppNetwork): a list of #MMModem3gppNetwork structs, or #NULL if @error is set. The returned value should be freed with g_list_free_full() using mm_modem_3gpp_network_free() as #GDestroyNotify function.
  */
 GList *
 mm_modem_3gpp_scan_finish (MMModem3gpp *self,
@@ -552,7 +571,7 @@ mm_modem_3gpp_scan (MMModem3gpp *self,
  * The calling thread is blocked until a reply is received. See mm_modem_3gpp_scan()
  * for the asynchronous version of this method.
  *
- * Returns: a list of #MMModem3gppNetwork structs, or #NULL if @error is set. The returned value should be freed with g_list_free_full() using mm_modem_3gpp_network_free() as #GDestroyNotify function.
+ * Returns:  (transfer full) (element-type ModemManager.Modem3gppNetwork):  a list of #MMModem3gppNetwork structs, or #NULL if @error is set. The returned value should be freed with g_list_free_full() using mm_modem_3gpp_network_free() as #GDestroyNotify function.
  */
 GList *
 mm_modem_3gpp_scan_sync (MMModem3gpp *self,
