@@ -195,6 +195,7 @@ nwrat_query_ready (MMBaseModem *self,
                                              response);
         g_simple_async_result_complete (simple);
         g_object_unref (simple);
+        g_match_info_free (match_info);
         g_regex_unref (r);
         return;
     }
@@ -406,7 +407,7 @@ typedef struct {
 
 typedef struct {
     MMBaseModem *self;
-    MMQcdmSerialPort *port;
+    MMPortSerialQcdm *port;
     GSimpleAsyncResult *simple;
     MMModemAccessTechnology generic_act;
     guint mask;
@@ -451,24 +452,27 @@ snapshot_context_complete_and_free (SnapshotContext *ctx, guint hdr_revision)
 }
 
 static void
-nw_snapshot_old_cb (MMQcdmSerialPort *port,
-                    GByteArray *response,
-                    GError *error,
-                    gpointer user_data)
+nw_snapshot_old_cb (MMPortSerialQcdm *port,
+                    GAsyncResult *res,
+                    SnapshotContext *ctx)
 {
-    SnapshotContext *ctx = user_data;
     QcdmResult *result;
     guint8 hdr_revision = QCDM_HDR_REV_UNKNOWN;
+    GError *error = NULL;
+    GByteArray *response;
 
+    response = mm_port_serial_qcdm_command_finish (port, res, &error);
     if (error) {
         /* Just ignore the error and complete with the input info */
         mm_dbg ("Couldn't run QCDM Novatel Modem MSM6500 snapshot: '%s'", error->message);
+        g_error_free (error);
         snapshot_context_complete_and_free (ctx, QCDM_HDR_REV_UNKNOWN);
         return;
     }
 
     /* Parse the response */
     result = qcdm_cmd_nw_subsys_modem_snapshot_cdma_result ((const gchar *) response->data, response->len, NULL);
+    g_byte_array_unref (response);
     if (result) {
         qcdm_result_get_u8 (result, QCDM_CMD_NW_SUBSYS_MODEM_SNAPSHOT_CDMA_ITEM_HDR_REV, &hdr_revision);
         qcdm_result_unref (result);
@@ -479,24 +483,27 @@ nw_snapshot_old_cb (MMQcdmSerialPort *port,
 }
 
 static void
-nw_snapshot_new_cb (MMQcdmSerialPort *port,
-                    GByteArray *response,
-                    GError *error,
-                    gpointer user_data)
+nw_snapshot_new_cb (MMPortSerialQcdm *port,
+                    GAsyncResult *res,
+                    SnapshotContext *ctx)
 {
-    SnapshotContext *ctx = user_data;
     QcdmResult *result;
     GByteArray *nwsnap;
     guint8 hdr_revision = QCDM_HDR_REV_UNKNOWN;
+    GError *error = NULL;
+    GByteArray *response;
 
+    response = mm_port_serial_qcdm_command_finish (port, res, &error);
     if (error) {
         mm_dbg ("Couldn't run QCDM Novatel Modem MSM6800 snapshot: '%s'", error->message);
+        g_error_free (error);
         snapshot_context_complete_and_free (ctx, QCDM_HDR_REV_UNKNOWN);
         return;
     }
 
     /* Parse the response */
     result = qcdm_cmd_nw_subsys_modem_snapshot_cdma_result ((const gchar *) response->data, response->len, NULL);
+    g_byte_array_unref (response);
     if (result) {
         qcdm_result_get_u8 (result, QCDM_CMD_NW_SUBSYS_MODEM_SNAPSHOT_CDMA_ITEM_HDR_REV, &hdr_revision);
         qcdm_result_unref (result);
@@ -510,7 +517,13 @@ nw_snapshot_new_cb (MMQcdmSerialPort *port,
     nwsnap = g_byte_array_sized_new (25);
     nwsnap->len = qcdm_cmd_nw_subsys_modem_snapshot_cdma_new ((char *) nwsnap->data, 25, QCDM_NW_CHIPSET_6500);
     g_assert (nwsnap->len);
-    mm_qcdm_serial_port_queue_command (port, nwsnap, 3, NULL, nw_snapshot_old_cb, ctx);
+    mm_port_serial_qcdm_command (port,
+                                 nwsnap,
+                                 3,
+                                 NULL,
+                                 (GAsyncReadyCallback)nw_snapshot_old_cb,
+                                 ctx);
+    g_byte_array_unref (nwsnap);
 }
 
 static gboolean
@@ -521,7 +534,7 @@ get_nw_modem_snapshot (MMBaseModem *self,
 {
     SnapshotContext *ctx;
     GByteArray *nwsnap;
-    MMQcdmSerialPort *port;
+    MMPortSerialQcdm *port;
 
     port = mm_base_modem_peek_port_qcdm (self);
     if (!port)
@@ -539,7 +552,13 @@ get_nw_modem_snapshot (MMBaseModem *self,
     nwsnap = g_byte_array_sized_new (25);
     nwsnap->len = qcdm_cmd_nw_subsys_modem_snapshot_cdma_new ((char *) nwsnap->data, 25, QCDM_NW_CHIPSET_6800);
     g_assert (nwsnap->len);
-    mm_qcdm_serial_port_queue_command (port, nwsnap, 3, NULL, nw_snapshot_new_cb, ctx);
+    mm_port_serial_qcdm_command (port,
+                                 nwsnap,
+                                 3,
+                                 NULL,
+                                 (GAsyncReadyCallback)nw_snapshot_new_cb,
+                                 ctx);
+    g_byte_array_unref (nwsnap);
     return TRUE;
 }
 
@@ -797,6 +816,8 @@ nwrssi_ready (MMBaseModem *self,
         quality = get_one_quality (response, "1x RSSI=");
     if (quality < 0)
         quality = get_one_quality (response, "RX1=");
+    if (quality < 0)
+        quality = get_one_quality (response, "HDR RSSI=");
 
     if (quality >= 0)
         g_simple_async_result_set_op_res_gpointer (simple,
@@ -966,20 +987,23 @@ parse_modem_eri (DetailedRegistrationStateContext *ctx, QcdmResult *result)
 }
 
 static void
-reg_eri_6500_cb (MMQcdmSerialPort *port,
-                 GByteArray *response,
-                 GError *error,
-                 gpointer user_data)
+reg_eri_6500_cb (MMPortSerialQcdm *port,
+                 GAsyncResult *res,
+                 DetailedRegistrationStateContext *ctx)
 {
-    DetailedRegistrationStateContext *ctx = user_data;
+    GError *error = NULL;
+    GByteArray *response;
 
+    response = mm_port_serial_qcdm_command_finish (port, res, &error);
     if (error) {
         /* Just ignore the error and complete with the input info */
         mm_dbg ("Couldn't run QCDM MSM6500 ERI: '%s'", error->message);
+        g_error_free (error);
     } else {
         QcdmResult *result;
 
         result = qcdm_cmd_nw_subsys_eri_result ((const gchar *) response->data, response->len, NULL);
+        g_byte_array_unref (response);
         if (result) {
             parse_modem_eri (ctx, result);
             qcdm_result_unref (result);
@@ -992,31 +1016,41 @@ reg_eri_6500_cb (MMQcdmSerialPort *port,
 }
 
 static void
-reg_eri_6800_cb (MMQcdmSerialPort *port,
-                 GByteArray *response,
-                 GError *error,
-                 gpointer user_data)
+reg_eri_6800_cb (MMPortSerialQcdm *port,
+                 GAsyncResult *res,
+                 DetailedRegistrationStateContext *ctx)
 {
-    DetailedRegistrationStateContext *ctx = user_data;
+    GError *error = NULL;
+    GByteArray *response;
 
+    response = mm_port_serial_qcdm_command_finish (port, res, &error);
     if (error) {
         /* Just ignore the error and complete with the input info */
         mm_dbg ("Couldn't run QCDM MSM6800 ERI: '%s'", error->message);
+        g_error_free (error);
     } else {
         QcdmResult *result;
-        GByteArray *nweri;
 
         /* Parse the response */
         result = qcdm_cmd_nw_subsys_eri_result ((const gchar *) response->data, response->len, NULL);
+        g_byte_array_unref (response);
         if (result) {
             parse_modem_eri (ctx, result);
             qcdm_result_unref (result);
         } else {
+            GByteArray *nweri;
+
             /* Try for MSM6500 */
             nweri = g_byte_array_sized_new (25);
             nweri->len = qcdm_cmd_nw_subsys_eri_new ((char *) nweri->data, 25, QCDM_NW_CHIPSET_6500);
             g_assert (nweri->len);
-            mm_qcdm_serial_port_queue_command (port, nweri, 3, NULL, reg_eri_6500_cb, ctx);
+            mm_port_serial_qcdm_command (port,
+                                         nweri,
+                                         3,
+                                         NULL,
+                                         (GAsyncReadyCallback)reg_eri_6500_cb,
+                                         ctx);
+            g_byte_array_unref (nweri);
             return;
         }
     }
@@ -1035,7 +1069,7 @@ modem_cdma_get_detailed_registration_state (MMIfaceModemCdma *self,
 {
     DetailedRegistrationStateContext *ctx;
     GByteArray *nweri;
-    MMQcdmSerialPort *port;
+    MMPortSerialQcdm *port;
 
     /* Setup context */
     ctx = g_new0 (DetailedRegistrationStateContext, 1);
@@ -1053,7 +1087,13 @@ modem_cdma_get_detailed_registration_state (MMIfaceModemCdma *self,
     nweri = g_byte_array_sized_new (25);
     nweri->len = qcdm_cmd_nw_subsys_eri_new ((char *) nweri->data, 25, QCDM_NW_CHIPSET_6800);
     g_assert (nweri->len);
-    mm_qcdm_serial_port_queue_command (port, nweri, 3, NULL, reg_eri_6800_cb, ctx);
+    mm_port_serial_qcdm_command (port,
+                                 nweri,
+                                 3,
+                                 NULL,
+                                 (GAsyncReadyCallback)reg_eri_6800_cb,
+                                 ctx);
+    g_byte_array_unref (nweri);
 }
 
 /*****************************************************************************/
