@@ -28,7 +28,7 @@
 #include "mm-log.h"
 #include "mm-sim-mbim.h"
 
-G_DEFINE_TYPE (MMSimMbim, mm_sim_mbim, MM_TYPE_SIM)
+G_DEFINE_TYPE (MMSimMbim, mm_sim_mbim, MM_TYPE_BASE_SIM)
 
 /*****************************************************************************/
 
@@ -39,10 +39,10 @@ peek_device (gpointer self,
              gpointer user_data)
 {
     MMBaseModem *modem = NULL;
-    MMMbimPort *port;
+    MMPortMbim *port;
 
     g_object_get (G_OBJECT (self),
-                  MM_SIM_MODEM, &modem,
+                  MM_BASE_SIM_MODEM, &modem,
                   NULL);
     g_assert (MM_IS_BASE_MODEM (modem));
 
@@ -59,7 +59,7 @@ peek_device (gpointer self,
         return FALSE;
     }
 
-    *o_device = mm_mbim_port_peek_device (port);
+    *o_device = mm_port_mbim_peek_device (port);
     return TRUE;
 }
 
@@ -67,7 +67,7 @@ peek_device (gpointer self,
 /* Load SIM identifier */
 
 static gchar *
-load_sim_identifier_finish (MMSim *self,
+load_sim_identifier_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
@@ -108,7 +108,7 @@ simid_subscriber_ready_state_ready (MbimDevice *device,
 }
 
 static void
-load_sim_identifier (MMSim *self,
+load_sim_identifier (MMBaseSim *self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
@@ -135,7 +135,7 @@ load_sim_identifier (MMSim *self,
 /* Load IMSI */
 
 static gchar *
-load_imsi_finish (MMSim *self,
+load_imsi_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
@@ -176,7 +176,7 @@ imsi_subscriber_ready_state_ready (MbimDevice *device,
 }
 
 static void
-load_imsi (MMSim *self,
+load_imsi (MMBaseSim *self,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
@@ -203,7 +203,7 @@ load_imsi (MMSim *self,
 /* Load operator identifier */
 
 static gchar *
-load_operator_identifier_finish (MMSim *self,
+load_operator_identifier_finish (MMBaseSim *self,
                                  GAsyncResult *res,
                                  GError **error)
 {
@@ -243,7 +243,7 @@ load_operator_identifier_ready (MbimDevice *device,
 }
 
 static void
-load_operator_identifier (MMSim *self,
+load_operator_identifier (MMBaseSim *self,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
@@ -270,7 +270,7 @@ load_operator_identifier (MMSim *self,
 /* Load operator name */
 
 static gchar *
-load_operator_name_finish (MMSim *self,
+load_operator_name_finish (MMBaseSim *self,
                            GAsyncResult *res,
                            GError **error)
 {
@@ -310,7 +310,7 @@ load_operator_name_ready (MbimDevice *device,
 }
 
 static void
-load_operator_name (MMSim *self,
+load_operator_name (MMBaseSim *self,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
@@ -337,7 +337,7 @@ load_operator_name (MMSim *self,
 /* Send PIN */
 
 static gboolean
-send_pin_finish (MMSim *self,
+send_pin_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -386,7 +386,7 @@ pin_set_enter_ready (MbimDevice *device,
 }
 
 static void
-send_pin (MMSim *self,
+send_pin (MMBaseSim *self,
           const gchar *pin,
           GAsyncReadyCallback callback,
           gpointer user_data)
@@ -428,7 +428,7 @@ send_pin (MMSim *self,
 /* Send PUK */
 
 static gboolean
-send_puk_finish (MMSim *self,
+send_puk_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -478,7 +478,7 @@ puk_set_enter_ready (MbimDevice *device,
 }
 
 static void
-send_puk (MMSim *self,
+send_puk (MMBaseSim *self,
           const gchar *puk,
           const gchar *new_pin,
           GAsyncReadyCallback callback,
@@ -521,7 +521,7 @@ send_puk (MMSim *self,
 /* Enable PIN */
 
 static gboolean
-enable_pin_finish (MMSim *self,
+enable_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -558,7 +558,7 @@ pin_set_enable_ready (MbimDevice *device,
 }
 
 static void
-enable_pin (MMSim *self,
+enable_pin (MMBaseSim *self,
             const gchar *pin,
             gboolean enabled,
             GAsyncReadyCallback callback,
@@ -601,7 +601,7 @@ enable_pin (MMSim *self,
 /* Change PIN */
 
 static gboolean
-change_pin_finish (MMSim *self,
+change_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -638,7 +638,7 @@ pin_set_change_ready (MbimDevice *device,
 }
 
 static void
-change_pin (MMSim *self,
+change_pin (MMBaseSim *self,
             const gchar *old_pin,
             const gchar *new_pin,
             GAsyncReadyCallback callback,
@@ -679,7 +679,7 @@ change_pin (MMSim *self,
 
 /*****************************************************************************/
 
-MMSim *
+MMBaseSim *
 mm_sim_mbim_new_finish (GAsyncResult  *res,
                         GError       **error)
 {
@@ -694,9 +694,9 @@ mm_sim_mbim_new_finish (GAsyncResult  *res,
         return NULL;
 
     /* Only export valid SIMs */
-    mm_sim_export (MM_SIM (sim));
+    mm_base_sim_export (MM_BASE_SIM (sim));
 
-    return MM_SIM (sim);
+    return MM_BASE_SIM (sim);
 }
 
 void
@@ -710,7 +710,7 @@ mm_sim_mbim_new (MMBaseModem *modem,
                                 cancellable,
                                 callback,
                                 user_data,
-                                MM_SIM_MODEM, modem,
+                                MM_BASE_SIM_MODEM, modem,
                                 NULL);
 }
 
@@ -722,22 +722,22 @@ mm_sim_mbim_init (MMSimMbim *self)
 static void
 mm_sim_mbim_class_init (MMSimMbimClass *klass)
 {
-    MMSimClass *sim_class = MM_SIM_CLASS (klass);
+    MMBaseSimClass *base_sim_class = MM_BASE_SIM_CLASS (klass);
 
-    sim_class->load_sim_identifier = load_sim_identifier;
-    sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
-    sim_class->load_imsi = load_imsi;
-    sim_class->load_imsi_finish = load_imsi_finish;
-    sim_class->load_operator_identifier = load_operator_identifier;
-    sim_class->load_operator_identifier_finish = load_operator_identifier_finish;
-    sim_class->load_operator_name = load_operator_name;
-    sim_class->load_operator_name_finish = load_operator_name_finish;
-    sim_class->send_pin = send_pin;
-    sim_class->send_pin_finish = send_pin_finish;
-    sim_class->send_puk = send_puk;
-    sim_class->send_puk_finish = send_puk_finish;
-    sim_class->enable_pin = enable_pin;
-    sim_class->enable_pin_finish = enable_pin_finish;
-    sim_class->change_pin = change_pin;
-    sim_class->change_pin_finish = change_pin_finish;
+    base_sim_class->load_sim_identifier = load_sim_identifier;
+    base_sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
+    base_sim_class->load_imsi = load_imsi;
+    base_sim_class->load_imsi_finish = load_imsi_finish;
+    base_sim_class->load_operator_identifier = load_operator_identifier;
+    base_sim_class->load_operator_identifier_finish = load_operator_identifier_finish;
+    base_sim_class->load_operator_name = load_operator_name;
+    base_sim_class->load_operator_name_finish = load_operator_name_finish;
+    base_sim_class->send_pin = send_pin;
+    base_sim_class->send_pin_finish = send_pin_finish;
+    base_sim_class->send_puk = send_puk;
+    base_sim_class->send_puk_finish = send_puk_finish;
+    base_sim_class->enable_pin = enable_pin;
+    base_sim_class->enable_pin_finish = enable_pin_finish;
+    base_sim_class->change_pin = change_pin;
+    base_sim_class->change_pin_finish = change_pin_finish;
 }
